hmscH.HMSCdata <-
function(data,start,priors=NULL,niter=12000,nburn=2000,nrot=100,thin=1,likelihoodtype="logit",spAsso=FALSE,details=TRUE,verbose=TRUE,...){
#### F. Guillaume Blanchet - Janvier 2013, March 2013, April 2013
##########################################################################################
	call<-match.call()
	
	### Basic objects
	nparamX<-ncol(start$paramX)
	nsp<-nrow(start$paramX)
	
	### General checks
	if(niter < nburn){
		stop("'niter' should be equal or larger than 'burning'")
	}
	
	if(is.null(priors)){
		if(!any(class(data)=="HMSCdataRandom")){
			priors<-list(means0=rep(0,nparamX),kappa0=0,nu0=rep(0,nparamX),Lambda0=diag(nparamX))
		}else{
			priors<-list(means0=rep(0,nparamX),kappa0=0,nu0=rep(0,nparamX),Lambda0=diag(nparamX),priorRandom=c(-1,1))
			nparamRandom<-ncol(start$paramRandom)
		}
	}
	
	#=====================================================================================================
	#### Construct the model using Single Component Adaptative Metropolis (SCAM) with eigenvector rotation
	#=====================================================================================================
	#--------------------
	#### Starting objects
	#--------------------
	#### Rotation objects (to use when no rotation is carried out)
	if(!any(class(data)=="HMSCdataRandom")){
		rotRes<-vector("list",length=4)
		names(rotRes)<-c("values","vectors","wParamX","wOuterParamX")
		rotRes[[1]]<-matrix(1,nrow=nsp,ncol=nparamX)
		rotRes[[2]]<-array(diag(nparamX),dim=c(nparamX,nparamX,nsp))
		rotRes[[3]]<-matrix(0,nrow=nsp,ncol=nparamX)
		rotRes[[4]]<-array(0,dim=c(nparamX,nparamX,nsp))
	}else{
		rotRes<-vector("list",length=9)
		names(rotRes)<-c("values","vectors","wParamX","wOuterParamX","valuesRandom","vectorsRandom","wParamRandom","wOuterParamRandom","wRandomCov")
		rotRes[[1]]<-matrix(1,nrow=nsp,ncol=nparamX)
		rotRes[[2]]<-array(diag(nparamX),dim=c(nparamX,nparamX,nsp))
		rotRes[[3]]<-matrix(0,nrow=nsp,ncol=nparamX)
		rotRes[[4]]<-array(0,dim=c(nparamX,nparamX,nsp))

		rotRes[[5]]<-matrix(1,nrow=nsp,ncol=nparamRandom)
		rotRes[[6]]<-array(diag(nparamRandom),dim=c(nparamRandom,nparamRandom,nsp))
		rotRes[[7]]<-matrix(0,nrow=nsp,ncol=nparamRandom)
		rotRes[[8]]<-array(0,dim=c(nparamRandom,nparamRandom,nsp))

		rotRes[[9]]<-1
	}
	
	if(!any(class(data)=="HMSCdataRandom")){
		accept<-array(0,dim=c(nsp,nparamX,2))
		sdvalue<-matrix(1,nrow=nsp,ncol=nparamX)
	}else{
		acceptparamX<-array(0,dim=c(nsp,nparamX,2))
		acceptparamRandom<-array(0,dim=c(nsp,nparamRandom,2))
		acceptRandomCov<-c(0,0)
		accept<-list(accept=acceptparamX,acceptRandom=acceptparamRandom,acceptRandomCov=acceptRandomCov)
		
		sdvalueParamX<-matrix(1,nrow=nsp,ncol=nparamX)
		sdvalueParamRandom<-matrix(1,nrow=nsp,ncol=nparamRandom)
		sdvalueRandomCov<-1
		sdvalue<-list(sdvalue=sdvalueParamX,sdvalueRandom=sdvalueParamRandom,sdvalueRandomCov=sdvalueRandomCov)
	}
	scamRes<-list(start=start,rotation=rotRes,sdvalue=sdvalue,accept=accept)

	#### Store burning results
	if(!any(class(data)=="HMSCdataRandom")){
		if(!spAsso){
			burningObj<-vector("list",length=3)
			names(burningObj)<-c("paramX","means","sigma")
			burningObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
			burningObj[[2]]<-matrix(NA,nrow=nburn,ncol=nparamX)
			burningObj[[3]]<-array(dim=c(nparamX,nparamX,nburn))
		}else{
			burningObj<-vector("list",length=4)
			names(burningObj)<-c("paramX","means","sigma","R")
			burningObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
			burningObj[[2]]<-matrix(NA,nrow=nburn,ncol=nparamX)
			burningObj[[3]]<-array(dim=c(nparamX,nparamX,nburn))
			burningObj[[4]]<-array(dim=c(nsp,nsp,nburn))
		}
	}else{
		if(!spAsso){
			burningObj<-vector("list",length=5)
			names(burningObj)<-c("paramX","means","sigma","paramRandom","RandomVar")
			burningObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
			burningObj[[2]]<-matrix(NA,nrow=nburn,ncol=nparamX)
			burningObj[[3]]<-array(dim=c(nparamX,nparamX,nburn))
			burningObj[[4]]<-array(dim=c(nsp,nparamRandom,nburn))
			burningObj[[5]]<-rep(NA,length=nburn)
		}else{
			burningObj<-vector("list",length=6)
			names(burningObj)<-c("paramX","means","sigma","paramRandom","RandomVar","R")
			burningObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
			burningObj[[2]]<-matrix(NA,nrow=nburn,ncol=nparamX)
			burningObj[[3]]<-array(dim=c(nparamX,nparamX,nburn))
			burningObj[[4]]<-array(dim=c(nsp,nparamRandom,nburn))
			burningObj[[5]]<-rep(NA,length=nburn)
			burningObj[[6]]<-array(dim=c(nsp,nsp,nburn))
		}
	}
	#### Store detailed information
	if(details){
		if(!any(class(data)=="HMSCdataRandom")){
			if(!spAsso){
				detailsObj<-vector("list",length=2)
				names(detailsObj)<-c("sdvalue","acceptRatio")
				detailsObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[1]][,,1]<-sdvalue
				detailsObj[[2]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[2]][,,1]<-matrix(0,nrow=nsp,ncol=nparamX)
			}else{
				detailsObj<-vector("list",length=3)
				names(detailsObj)<-c("sdvalue","acceptRatio","acceptRatioR")
				detailsObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[1]][,,1]<-sdvalue
				detailsObj[[2]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[2]][,,1]<-matrix(0,nrow=nsp,ncol=nparamX)
				detailsObj[[3]]<-numeric()
			}
		}else{
			if(!spAsso){
				detailsObj<-vector("list",length=6)
				names(detailsObj)<-c("sdvalue","acceptRatio","acceptRatioRandomCov","acceptRatioRandom","sdvalueRandom","sdvalueRandomCov")
				detailsObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[1]][,,1]<-sdvalueParamX
				detailsObj[[2]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[2]][,,1]<-matrix(0,nrow=nsp,ncol=nparamX)
				detailsObj[[3]]<-numeric()
				detailsObj[[3]][1]<-0
				detailsObj[[4]]<-array(dim=c(nsp,nparamRandom,nburn))
				detailsObj[[4]][,,1]<-matrix(0,nrow=nsp,ncol=nparamRandom)
				detailsObj[[5]]<-array(dim=c(nsp,nparamRandom,nburn))
				detailsObj[[5]][,,1]<-sdvalueParamRandom
				detailsObj[[6]]<-numeric()
				detailsObj[[6]][1]<-0
			}else{
				detailsObj<-vector("list",length=7)
				names(detailsObj)<-c("sdvalue","acceptRatio","acceptRatioR","acceptRatioRandomCov","acceptRatioRandom","sdvalueRandom","sdvalueRandomCov")
				detailsObj[[1]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[1]][,,1]<-sdvalueParamX
				detailsObj[[2]]<-array(dim=c(nsp,nparamX,nburn))
				detailsObj[[2]][,,1]<-matrix(0,nrow=nsp,ncol=nparamX)
				detailsObj[[3]]<-numeric()
				detailsObj[[3]][1]<-0
				detailsObj[[4]]<-numeric()
				detailsObj[[4]][1]<-0
				detailsObj[[5]]<-array(dim=c(nsp,nparamRandom,nburn))
				detailsObj[[5]][,,1]<-matrix(0,nrow=nsp,ncol=nparamRandom)
				detailsObj[[6]]<-array(dim=c(nsp,nparamRandom,nburn))
				detailsObj[[6]][,,1]<-sdvalueParamRandom
				detailsObj[[7]]<-numeric()
				detailsObj[[7]][1]<-0
			}
		}
	}
	
	#### Store results after burning
	if(!any(class(data)=="HMSCdataRandom")){
		if(!spAsso){
			modelres<-vector("list",length=5)
			names(modelres)<-c("paramX","means","sigma","sdvalue","acceptRatio")
			nvalues<-floor((niter-nburn)/thin)
			modelres[[1]]<-array(dim=c(nsp,nparamX,nvalues))
			modelres[[2]]<-matrix(NA,nrow=nvalues,ncol=nparamX)
			modelres[[3]]<-array(dim=c(nparamX,nparamX,nvalues))
		}else{
			modelres<-vector("list",length=7)
			names(modelres)<-c("paramX","means","sigma","sdvalue","acceptRatio","R","acceptRatioR")
			nvalues<-floor((niter-nburn)/thin)
			modelres[[1]]<-array(dim=c(nsp,nparamX,nvalues))
			modelres[[2]]<-matrix(NA,nrow=nvalues,ncol=nparamX)
			modelres[[3]]<-array(dim=c(nparamX,nparamX,nvalues))
			modelres[[6]]<-array(dim=c(nsp,nsp,nvalues))
			modelres[[7]]<-numeric()
		}
	}else{
		if(!spAsso){
			modelres<-vector("list",length=10)
			names(modelres)<-c("paramX","means","sigma","sdvalue","acceptRatio","paramRandom","sdvalueRandomCov","sdvalueRandom","acceptRatioRandomCov","acceptRatioRandom")
			nvalues<-floor((niter-nburn)/thin)
			modelres$paramX<-array(dim=c(nsp,nparamX,nvalues))
			modelres$means<-matrix(NA,nrow=nvalues,ncol=nparamX)
			modelres$sigma<-array(dim=c(nparamX,nparamX,nvalues))
			modelres$paramRandom<-array(dim=c(nsp,nparamRandom,nvalues))
		}else{
			modelres<-vector("list",length=12)
			names(modelres)<-c("paramX","means","sigma","sdvalue","acceptRatio","R","acceptRatioR","paramRandom","sdvalueRandomCov","sdvalueRandom","acceptRatioRandomCov","acceptRatioRandom")
			nvalues<-floor((niter-nburn)/thin)
			modelres$paramX<-array(dim=c(nsp,nparamX,nvalues))
			modelres$means<-matrix(NA,nrow=nvalues,ncol=nparamX)
			modelres$sigma<-array(dim=c(nparamX,nparamX,nvalues))
			modelres$R<-array(dim=c(nsp,nsp,nvalues))
			modelres$paramRandom<-array(dim=c(nsp,nparamRandom,nvalues))
		}
	}
	
	### Generate print counter
	if(verbose){
		printCounter<-round(seq(niter/5,niter,length=5))
	}
	#==================================
	#### SCAM with eigenvector rotation
	#==================================
	itsave<-1
	if(!any(class(data)=="HMSCdataRandom")){
		for(i in 1:niter){
			scamRes<-scamH(data,scamRes$start,priors,scamRes$rotation$values,scamRes$rotation$vectors,scamRes$rotation$wParamX,scamRes$rotation$wOuterParamX,scamRes$sdvalue,scamRes$accept,likelihoodtype=likelihoodtype,burning=(i < nburn),rotation=(i > nrot & i < nburn),iter=i)
			#-------------------------
			#### Store burning results
			#-------------------------
			if(i < nburn){
				burningObj$paramX[,,i]<-scamRes$start$paramX
				burningObj$means[i,]<-scamRes$start$means
				burningObj$sigma[,,i]<-scamRes$start$sigma
				
				if(details){
					detailsObj$sdvalue[,,i+1]<-scamRes$sdvalue
					detailsObj$acceptRatio[,,i+1]<-scamRes$accept[,,1]/scamRes$accept[,,2]
				}
			}else{
				it<-i-nburn
				if(it==thin*itsave){
					modelres$paramX[,,itsave]<-scamRes$start$paramX
					modelres$means[itsave,]<-scamRes$start$means
					modelres$sigma[,,itsave]<-scamRes$start$sigma
					itsave<-itsave+1
				}
			}
			
			### Species association matrix
			if(spAsso){
				### Ajouter sp
			}
			
			### Print number of iterations completed
			if(verbose){
				if(any(i==printCounter)){
					print(i)
				}
			}
		}
	}else{
		for(i in 1:niter){
			#---------------
			### Fixed effect
			#---------------
if(TRUE){
			scamResH<-scamH(data,scamRes$start,priors,scamRes$rotation$values,scamRes$rotation$vectors,scamRes$rotation$wParamX,scamRes$rotation$wOuterParamX,scamRes$sdvalue$sdvalue,scamRes$accept$accept,likelihoodtype=likelihoodtype,burning=(i < nburn),rotation=(i > nrot & i < nburn),iter=i) 
			
			### Assign the new estimated pieces of scamResH to scamRes
			scamRes$start$paramX<-scamResH$start$paramX
			scamRes$start$sigma<-scamResH$start$sigma
			scamRes$start$means<-scamResH$start$means
			
			scamRes$rotation$values<-scamResH$rotation$values
			scamRes$rotation$vectors<-scamResH$rotation$vectors
			scamRes$rotation$wParamX<-scamResH$rotation$wParamX
			scamRes$rotation$wOuterParamX<-scamResH$rotation$wOuterParamX
			
			scamRes$sdvalue$sdvalue<-scamResH$sdvalue
			scamRes$accept$accept<-scamResH$accept
}			
			#----------------
			### Random effect
			#----------------
			scamResRandom<-scamRandom(data,start=scamRes$start,priors=priors,rotationVal=scamRes$rotation$valuesRandom,rotationVect=scamRes$rotation$vectorsRandom,wParamRandom=scamRes$rotation$wParamRandom,wOuterParamRandom=scamRes$rotation$wOuterParamRandom,wRandomCov=scamRes$rotation$wRandomCov,sdvalue=scamRes$sdvalue,accept=scamRes$accept,likelihoodtype=likelihoodtype,burning=(i < nburn),rotation=(i > nrot & i < nburn),iter=i)
			
			### Assign the new estimated pieces of scamResH to scamRes
			scamRes$start$paramRandom<-scamResRandom$start$paramRandom
			scamRes$start$RandomVar<-scamResRandom$start$RandomVar
			
			scamRes$rotation$valuesRandom<-scamResRandom$rotation$valuesRandom
			scamRes$rotation$vectorsRandom<-scamResRandom$rotation$vectorsRandom
			scamRes$rotation$wParamRandom<-scamResRandom$rotation$wParamRandom
			scamRes$rotation$wOuterParamRandom<-scamResRandom$rotation$wOuterParamRandom
			scamRes$rotation$wRandomCov<-scamResRandom$rotation$wRandomCov
			
			scamRes$sdvalue<-scamResRandom$sdvalue
			scamRes$accept<-scamResRandom$accept
			
			#-------------------------
			#### Store burning results
			#-------------------------
			
			if(i < nburn){
				burningObj$paramX[,,i]<-scamRes$start$paramX
				burningObj$means[i,]<-scamRes$start$means
				burningObj$sigma[,,i]<-scamRes$start$sigma
				burningObj$paramRandom[,,i]<-scamRes$start$paramRandom
				burningObj$RandomVar[i]<-scamRes$start$RandomVar
				
				if(details){
					detailsObj$sdvalue[,,i+1]<-scamRes$sdvalue$sdvalue
					detailsObj$acceptRatio[,,i+1]<-scamRes$accept$accept[,,1]/scamRes$accept$accept[,,2]
					
					detailsObj$sdvalueRandomCov[i+1]<-scamRes$sdvalue$sdvalueRandomCov[1]
					detailsObj$acceptRatioRandomCov[i+1]<-scamRes$accept$acceptRandomCov[1]/scamRes$accept$acceptRandomCov[2]
					
					detailsObj$sdvalueRandom[,,i+1]<-scamRes$sdvalue$sdvalueRandom
					detailsObj$acceptRatioRandom[,,i+1]<-scamRes$accept$acceptRandom[,,1]/scamRes$accept$acceptRandom[,,2]
				}
			}else{
				it<-i-nburn
				if(it==thin*itsave){
					modelres$paramX[,,itsave]<-scamRes$start$paramX
					modelres$means[itsave,]<-scamRes$start$means
					modelres$sigma[,,itsave]<-scamRes$start$sigma
					modelres$paramRandom[,,itsave]<-scamRes$start$paramRandom
					modelres$RandomVar[itsave]<-scamRes$start$RandomVar
					itsave<-itsave+1
				}
			}
			
			### Species association matrix
			if(spAsso){
				### Ajouter sp
			}
			
			### Print number of iterations completed
			if(verbose){
				if(any(i==printCounter)){
					print(i)
				}
			}
		}
	}
	
	if(!any(class(data)=="HMSCdataRandom")){
		modelres$sdvalue<-scamRes$sdvalue
		modelres$acceptRatio<-scamRes$accept[,,1]/scamRes$accept[,,2]
	}else{
		modelres$sdvalue<-scamRes$sdvalue$sdvalue
		modelres$acceptRatio<-scamRes$accept$accept[,,1]/scamRes$accept$accept[,,2]
		
		modelres$sdvalueRandomCov<-scamRes$sdvalue$sdvalueRandomCov
		modelres$acceptRatioRandomCov<-scamRes$accept$acceptRandomCov[1]/scamRes$accept$acceptRandomCov[2]
		
		modelres$sdvalueRandom<-scamRes$sdvalue$sdvalueRandom
		modelres$acceptRatioRandom<-scamRes$accept$acceptRandom[,,1]/scamRes$accept$acceptRandom[,,2]
	}
	#===================
	#### Name the pieces
	#===================
	### paramX
	dimnames(modelres$paramX)[[1]]<-rownames(start$paramX)
	dimnames(modelres$paramX)[[2]]<-colnames(start$paramX)

	### means
	if(ncol(modelres$means)==1){
		modelres$means<-as.data.frame(modelres$means)
	}
	dimnames(modelres$means)[[2]]<-colnames(start$paramX)
	
	### sigma
	if(ncol(modelres$means)!=1){
		dimnames(modelres$sigma)[[1]]<-colnames(start$paramX)
		dimnames(modelres$sigma)[[2]]<-colnames(start$paramX)
	}
	
	if(any(class(data)=="HMSCdataRandom")){
		### paramRandom
		dimnames(modelres$paramRandom)[[1]]<-rownames(start$paramRandom)
		dimnames(modelres$paramRandom)[[2]]<-colnames(start$paramRandom)
	}
	
	#==========
	### Results
	#==========
	if(details){
		res<-list(call=call,model=modelres,burning=burningObj,details=detailsObj,data=data)
	}else{
		res<-list(call=call,model=modelres,burning=burningObj,data=data)
	}
	
	attr(res,"likelihood")<-likelihoodtype
	
	if(!any(class(data)=="HMSCdataRandom")){
		class(res)<-c("hmsc","HMSCdata")
	}else{
		class(res)<-c("hmsc","HMSCdata","HMSCdataRandom")
	}
	return(res)
}
