scamHT.HMSCdata <-
function(data,start,priors,rotationVal,rotationVect,wParamX,wOuterParamX,sdvalue,accept,likelihoodtype="logit",burning=TRUE,rotation=TRUE,iter,...){
#### F. Guillaume Blanchet - Janvier 2013, March 2013, April 2013, June 2013, August 2013
##########################################################################################
	#========================
	#### Initiation of object
	#========================
	nparamX<-ncol(start$paramX)
	nparamTr<-ncol(start$paramTr)
	nsp<-ncol(data$Y)
	nsite<-nrow(data$Y)
	
	paramXCurrent<-start$paramX
	paramTrCurrent<-start$paramTr
	means<-start$means
	sigma<-start$sigma
	
	#### Calculate log-likelihood - mean
	likeCurrentMean<-numeric()
	if(!any(class(data)=="HMSCdataRandom")){
		for(i in 1:nsp){
			likeCurrentMean[i]<-loglikelihood(data$Y[,i],data$X,paramXCurrent[i,],likelihoodtype=likelihoodtype)
		}
	}else{
		for(i in 1:nsp){
			likeCurrentMean[i]<-loglikelihood(data$Y[,i],data$X,paramXCurrent[i,],data$Random,start$paramRandom[i,],likelihoodtype=likelihoodtype)
		}
	}
	
	likeCurrentSigma<-numeric()
	for(i in 1:nsp){
		likeCurrentSigma[i]<-loglikelihood(sigma,paramXCurrent[i,],means[,i],likelihoodtype="sigma")
	}
	
	#=============
	#### Algorithm
	#=============
if(TRUE){
	#----------------------------------------
	### Update means (each one independently)
	#----------------------------------------
	paramXNew<-matrix(NA,nrow=nsp,ncol=nparamX)
#	paramXNew<-paramXCurrent
	for(k in 1:nsp){
		for(j in 1:nparamX){
			### Sample from a normal distribution and weight the sample by the eigenvalue of the covariance matrix
			paramXNew[k,]<-paramXCurrent[k,]+rnorm(1,mean=0,sd=sdvalue[k,j])*sqrt(rotationVal[k,j])*rotationVect[,j,k]
			
			#### Calculate log-likelihood - mean
			if(!any(class(data)=="HMSCdataRandom")){
				likeNewMean<-loglikelihood(data$Y[,k],data$X,paramXNew[k,],likelihoodtype=likelihoodtype)
			}else{
				likeNewMean<-loglikelihood(data$Y[,k],data$X,paramXNew[k,],data$Random,start$paramRandom[k,],likelihoodtype=likelihoodtype)
			}
			
			#### Calculate log-likelihood - sigma
			likeNewSigma <- loglikelihood(sigma,paramXNew[k,],means[,k],likelihoodtype="sigma")
			
			accept[k,j,2]<-accept[k,j,2]+1
			
			### Accept or reject the new paramX
			if(runif(1) <= exp(likeNewMean+likeNewSigma-likeCurrentMean[k]-likeCurrentSigma[k])){
				paramXCurrent[k,]<-paramXNew[k,]
				likeCurrentMean[k]<-likeNewMean
				likeCurrentSigma[k]<-likeNewSigma
				accept[k,j,1]<-accept[k,j,1]+1
			}
		}
	}
}	
	#--------------------------
	#### means and sigma update
	#--------------------------
if(TRUE){
	#_________________________________________________
	### Sample the parameters associated to the traits
	#_________________________________________________
	meansparamX<-colMeans(paramXCurrent)
	SigmaparamX<-crossprod(sweep(paramXCurrent,2,meansparamX,FUN="-"))
	
	### Calculate the precision (inverse of variance) of the multivariate normal distribution from which to sample parameters and the means (Note: meanSp = percSp%*%meansSpPart)
	nMeans<-nparamX*nparamTr
	Xtrait1Sp<-array(dim=c(nparamX,nMeans,nsp))
	
	precSp<-matrix(0,nrow=nMeans,ncol=nMeans)
	meansSpPart<-rep(0,length=nMeans)
	
	for(i in 1:nsp){
		Xtrait1SpBase<-t(kronecker(data$Tr[,i],diag(1,nparamX)))
		Xtrait1Sp[,,i]<-Xtrait1SpBase
		Xtrait1SpSigma<-crossprod(Xtrait1SpBase,chol2inv(chol(SigmaparamX)))
		precSp<-precSp+Xtrait1SpSigma%*%Xtrait1SpBase
		meansSpPart<-meansSpPart+Xtrait1SpSigma%*%paramXCurrent[i,]
	}
	
	meansSpBase<-mvrnorm(1,rep(0,nMeans),diag(nMeans))

	### Cholesky decomposition of precSp
	cholprecSp<-chol(precSp)
	
	### Calculate the new paramTrCurrent
	meansSpError<-backsolve(cholprecSp,meansSpBase)
	meansSpStruc<-crossprod(chol2inv(cholprecSp),meansSpPart)
	paramTrCurrent<-meansSpStruc+meansSpError
	paramTrCurrent<-drop(paramTrCurrent)
	#_________________________________
	### Sample the new variance matrix
	#_________________________________
	modelErr<-matrix(NA,nrow=nsp,ncol=nparamX)
	
	for(i in 1:nsp){
		modelErr[i,]<-paramXCurrent[i,]-Xtrait1Sp[,,i]%*%paramTrCurrent
	}
	
	modelErrMean<-colMeans(modelErr)
	kappaSp<-priors$kappa0+nsp
#	LambdaSp<-priors$Lambda0+SigmaparamX+crossprod(sweep(modelErr,2,modelErrMean,FUN="-"))
#	sigma<-solve(rWishart(1,nparamX+2+nsp,solve(LambdaSp))[,,1])
	LambdaSp<-priors$Lambda0+SigmaparamX+(priors$kappa0*nsp)/(kappaSp)*crossprod(sweep(modelErr,2,modelErrMean,FUN="-"))
	sigma<-solve(rWishart(1,nparamX+2+nsp,solve(LambdaSp))[,,1])
	
	### Calculate the mean of the multivariate normal distribution from which to sample
	dim(paramTrCurrent)<-c(nparamX,nparamTr)
	for(i in 1:nsp){
		means[,i]<-tcrossprod(data$Tr[,i],paramTrCurrent)
	}
}

if(TRUE){
	if(burning){
		wAccept<-1-0.1*exp(-iter/500) # w
		### Greediness during the adaptation period
		greedAdapt<-1+exp(-iter/500) # q
		
		### Calculate the accept ratio
		acceptRatio<-accept[,,1]/accept[,,2]
		
		### Modify the standard deviation
		sdvalue<-sdvalue*greedAdapt^(acceptRatio-0.44)
		
		### Weight the accept ratio
		accept<-accept*wAccept
				
		### Weight the current ParamX
		wParamX<-wParamX+wAccept*paramXCurrent # s1
		### Weight the outer product of the current ParamX
		for(j in 1:nsp){
			wOuterParamX[,,j]<-wOuterParamX[,,j]+wAccept*outer(paramXCurrent[j,],paramXCurrent[j,]) # s2
		}
		
		if(rotation){
			### sum the weight
			weight<-sum(1-0.1*exp(-(1:iter)/500))
			
			for(j in 1:nsp){
				### Construct a weighted covariance matrix
				covMat<-(wOuterParamX[,,j]-outer(wParamX[j,],wParamX[j,])/weight)/(weight-1)+diag(.Machine$double.eps^0.33,nparamX)
				rotationBase<-eigen(covMat)
				
				rotationVal[j,]<-rotationBase$values
				rotationVect[,,j]<-rotationBase$vectors
			}
		}
	}
	
}
	#### Result object
	colnames(sigma)<-rownames(means)
	rownames(sigma)<-rownames(means)
	startNew<-list(paramX=paramXCurrent,paramTr=paramTrCurrent,means=means,sigma=sigma)
	rotNew<-list(values=rotationVal,vectors=rotationVect,wParamX=wParamX,wOuterParamX=wOuterParamX)
	res<-list(start=startNew,rotation=rotNew,sdvalue=sdvalue,accept=accept)
	
	return(res)
}
