\name{basicStatHMSC}
\alias{basicStatHMSC}
\alias{summary.hmsc}
\alias{quantile.hmsc}

\title{
Summary statistics of the model probabilities on an object of class hmsc
}

\description{
The function \code{quantile} calculates quantiles on the probabilities of a model constructed with \code{\link{hmscH}}. Quantiles are calculated for each species included in the model.
The function \code{basicStatHMSC} calculates the means and standard deviations on the probabilities of a model constructed with \code{\link{hmscH}}.
The function \code{summary} calculates the means, standard deviations, and three quantiles (0.025,0.5,0.975) on the probabilities of a model constructed with \code{\link{hmscH}}.
}

\usage{
\method{summary}{hmsc}(object, ...)
\method{quantile}{hmsc}(x, probs = c(0.025, 0.5, 0.975), type = 8, ...)
basicStatHMSC(x, stats = c("mean", "sd"))
}

\arguments{
  \item{x}{
An object of class \code{\link{hmscH}}
An object of class hmsc
}
  \item{object}{
An object of class \code{\link{hmscH}}
}
  \item{probs}{
A numeric vector of probabilities with values ranging from 0 to 1.  See \code{\link{quantile}} for more details.
}
  \item{type}{
an integer between 1 and 9 selecting one of the nine quantile algorithms detailed in \code{\link{quantile}}. Default is 8 as suggested by Hyndman and Fan (1996).
}
  \item{stats}{
A character vector defining the statistics to calculate. Any unambiguous variation of wording used in this argument is accepted.
}
  \item{\dots}{
Arguments passed to \code{\link{quantile}} or to \code{\link{summary}}.
}
}

\details{
These functions can calculate summary statistics for any type of \code{\link{hmscH}} object, whether they stem from a linear or a non-linear model. 
}

\value{
For \code{quantile}: A three-dimensional array where the first dimension represents sites, the second dimension represents species, and the third dimensions the quantiles.
For \code{basicStatHMSC}: A matrix where the rows represents sites and the columns represents species if only the mean or the standard deviation is calculated. Otherwise, a three-dimensional array where the first dimension represents sites, the second dimension represents species, and the third dimensions the means and standard deviations.
}

\references{
Hyndman, R. J. and Fan, Y. (1996) Sample quantiles in statistical packages, \emph{American Statistician}, \strong{50}, 361-365.
}

\author{
F. Guillaume Blanchet
}
\seealso{
\code{\link{quantile}}, \code{\link{hmscH}}
}
\examples{
#==================
### Simulating data
#==================
nspecies<-20
desc<-cbind(1,scale(1:50),scale(1:50)^2)
dataBase<-communitySimulH(desc,nsp=nspecies)

#=================================
### Formatting data and parameters
#=================================
formdata<-as.HMSCdata(dataBase$data$Y,desc,Ypattern="sp",interceptX=FALSE,interceptTr=FALSE)

#========================
### Formatting parameters
#========================
startParamX<-matrix(rnorm(nspecies*ncol(desc)),nrow=nspecies,ncol=ncol(desc))
startMean<-rnorm(ncol(desc))
startSigma<-rWishart(1,ncol(desc)+1,diag(ncol(desc)))[,,1]

formparam<-as.HMSCparam(formdata,paramX=startParamX,means=startMean,sigma=startSigma)
formpriors<-as.HMSCprior(formparam,rep(0,length(formparam$means)),0,length(formparam$means),diag(length(formparam$means)))

#===================================================
### Building model using a linear modelling approach
#===================================================
modelLinear<-hmscH(formdata,formparam,formpriors,niter=200,nburn=100)

meansdHMSC<-basicStatHMSC(modelLinear)
quantileHMSC<-quantile(modelLinear)
summaryHMSC<-summary(modelLinear)

}
\keyword{ univar }
