\name{loglikelihood}
\alias{loglikelihood}
\alias{loglikelihood.formula}
\alias{loglikelihood.default}
\alias{loglikelihood.matrix}
\title{
Log-likelihood of a regression model
}
\description{
These functions calculate different log-likelihoods in the context of regression analyses. The different log-likelihoods are all calculated by using the function \code{loglikelihood}. Depending on the information that serves as input in the function different log-likelihood will be calculated. In the usage section, the first two ways to use \code{loglikelihood} allows the user to calculate the log-likelihood of the mean regression parameters whereas the third way calculates the log-likelihood of the sigma (the models variance-covariance matrix).
}
\usage{
\method{loglikelihood}{formula}(formula,paramX,data,likelihoodtype="logit",..., prior)
\method{loglikelihood}{default}(y, X, paramX,Random=NULL,paramRandom=NULL, likelihoodtype = "logit",..., prior)
\method{loglikelihood}{matrix}(mat,param,means,likelihoodtype="sigma",...)
}

\arguments{
  \item{formula}{
Model formula. The left side presents the response matrix and the right side gives the descriptors with their parameters. (See details)
}
  \item{y}{
Vector defining a single response variable.
}
  \item{X}{
Matrix where the descriptors are columns.
}
  \item{paramX}{
Vector of regression parameters associated to the fixed effect (X).
}
  \item{param}{
Vector of regression parameters. (May potentially change !)
}
  \item{Random}{
Matrix defining the random effect. Each column contains either 1s (effect) or 0s (absence of effect) of the random effect.
}
  \item{paramRandom}{
Vector of regression parameters associated to the random effect.
}

  \item{data}{
A data.frame including the variables used to calculate the log-likelihood.
}
  \item{mat}{
A symmetric positive definite matrix.
}
  \item{means}{
Vector of the regression parameter averaged over the response variables. This vectors should have the same length as \code{param}.
}
  \item{likelihoodtype}{
Character string defining how the likelihood should be calculated. Any unambiguous variation of wording used in this argument is accepted. (See details)
}
  \item{...}{
Parameters passed to other functions.
}
  \item{prior}{
Numeric. A value defining a prior for specific likelihood functions (e.g. Normal likelihood). (See details)
}  
}
\details{
The \code{formula} was designed so that it is necessary to include the parameters and the descriptors. This function is thus capable of carrying out non-linear regression analyses. 

The \code{prior} is used only in likelihood functions that require additional prior information for the estimation of the likelihood. For example, when estimating the likelihood of the mean of a normal distribution, prior information on the variance of the normal distribution is required.

So far, the following log-likelihood have been implemented. In the formulas bellow, the inverse link functions are represented by "l", the model is presented by "m", the response variable (typically a single species) is defined by "y" and "n" defines the number of samples:
	\tabular{ll}{
		\code{logit}
		\tab \eqn{\sum_{i=1}^n \log((l_i \times y_i)+((1-l_i)\times(1-y_i)))}{sum(log((l*y)+((1-l)*(1-y))))}
		\cr
		\code{Poisson}
		\tab \eqn{\sum_{i=1}^n (\log(l_i) \times y_i)-n \times l_i}{sum(log(l)*y)-n*l}
		\cr
		\code{normal}
		\tab \eqn{-1/(2*\sigma^2) \sum_{i=1}^n (y_i \times m_i)^2}{-1/(2*sigma^2) * sum((y-model)^2)}
		\cr
		\code{sigma}
		\tab \eqn{\frac{-(\beta_i -\bar{\beta_j}_i) \sigma^{-i}(\beta_i -\bar{\beta_j}_i)}{2}}{-(\beta_i -\bar{\beta_j}_i) \sigma^{-i}(\beta_i -\bar{\beta_j}_i)/2}
		\cr
		\code{R}
		\tab \eqn{-(p/2)\log(|R|)-\frac{\sum_{i=1}^p W_i R^{-1} W_i}{2}-\frac{p+1}{2}\log(|W|)-Tr(W^{-1})/2}{-(p/2)*log(det(R))-(sum_{i=1}^p W_i R^{-1} W_i)(2)-(p+1)/2*log(det(W))-sum(diag(W^{-1}))/2
		}
	}

For the sigma and R likelihood, the input needs to be a matrix. Note that the R likelihood used is designed for each non-diagonal values in the matrix to follow a uniform distribution ranging from -1 to 1.
}
\value{
A vector of log-likelihood values.
}
\author{
F. Guillaume Blanchet
}
\note{
These functions are meant to be used within \code{scam}. 
}

\seealso{
\code{\link{ilogit}}
}