\name{scam}
\alias{scamH}
\alias{scamH.formula}
\alias{scamH.HMSCdata}
\alias{scamHT}
\alias{scamHT.HMSCdata}
\alias{scamRandom}
\alias{scamRandom.HMSCdata}
\title{
Single Component Adaptative Metropolis (SCAM) algorithm with eigenvector rotation
}
\description{
Performs one iteration of the single component adaptative Metropolis (SCAM) algorithm with eigenvector rotation. \code{scamH} is designed to apply the SCAM algorithm for models where a species community is modelled using habitat characteristics whereas \code{scamHT} applies the SCAM algorithm to model a community with habitat characteristics and species traits.
}
\usage{
\method{scamH}{formula}(formula,data,start,priors,rotationVal,rotationVect,wParamX,wOuterParamX,sdvalue,accept,likelihoodtype="logit",burning=TRUE,rotation=TRUE,iter,...)

\method{scamH}{HMSCdata}(data,start,priors,rotationVal,rotationVect,wParamX,wOuterParamX,sdvalue,accept,likelihoodtype="logit",burning=TRUE,rotation=TRUE,iter,...)

\method{scamHT}{HMSCdata}(data,start,priors,rotationVal,rotationVect,wParamX,wOuterParamX,sdvalue,accept,likelihoodtype="logit",burning=TRUE,rotation=TRUE,iter,...)

\method{scamRandom}{HMSCdata}(data,start,priors,rotationVal,rotationVect,wParamRandom,wOuterParamRandom,wRandomCov,sdvalue,accept,likelihoodtype="logit",burning=TRUE,rotation=TRUE,iter,...)
}
\arguments{
  \item{formula}{
Model formula. The left side presents the response matrix and the right side gives the descriptors with their parameters. (See details)
}
  \item{data}{
Object of class HMSCdata.
}
  \item{start}{
Object of class HMSCparam.
}
  \item{priors}{
Object of class HMSCprior.
}
  \item{rotationVal}{
Matrix of eigenvalues independently weighting the importance of each parameter for a response variables. The matrix has as many rows as there are response variables and as many columns as parameters.
}
  \item{rotationVect}{
3-dimensional array where the first two dimensions present eigenvectors (indexed as the second dimension) for each response variables (third dimension).
}
  \item{wParamX}{
Matrix of the same dimensions as \code{HMSCparam$paramX} where the values define the weights each parameter should have when performing the eigenvector rotation. 
}
  \item{wOuterParamX}{
3-dimensional array presenting covariance matrices (the first two dimensions are the rows and columns of the covariance matrix and the third dimension presents the different covariance matrices) used to weight the importance of the parameters for each response variable.
}
  \item{wParamRandom}{
Matrix of the same dimensions as \code{HMSCparam$paramRandom} where the values define the weights each random effect parameter should have when performing the eigenvector rotation. 
}
  \item{wOuterParamRandom}{
3-dimensional array presenting covariance matrices (the first two dimensions are the rows and columns of the covariance matrix and the third dimension presents the different covariance matrices) used to weight the importance of the random effect parameters for each response variable.
}
  \item{wRandomCov}{
A scalar weighting of the importance of \code{RandomCov} when estimating the random effect parameters.
}
  \item{sdvalue}{
For \code{scamH} and \code{scamHT}, it is a matrix of standard deviation used when sampling for the regression parameters (columns) of each response variables (row). For \code{scamRandom}, it is a list with three components. The first component is a matrix of standard deviation used when sampling for the regression parameters (columns) of each response variables (row), the second component is a matrix of standard deviation used when sampling for the regression parameters for each levels of the random effect (columns) for each response variables (row) and the third component is a scalare used when sampling for \code{RandomVar}.
}
  \item{accept}{
For \code{scamH} and \code{scamHT}, it is 3-dimensional array where the first dimension is the number of parameters, the second dimension is is the number of response variables and the third dimension is 2. for \code{scamRandom}, it is a list with three components. The first component is a 3-dimensional array where the first dimension is the number of parameters, the second dimension is is the number of response variables and the third dimension is 2; the second component is a 3-dimensional array where the first dimension is the number of parameters for each levels of the random effect (columns), the second dimension is the number of response variables and the third dimension is 2; and the third component is a vector of length 2. 
}
  \item{likelihoodtype}{
Character string defining how the likelihood should be calculated. Any unambiguous variation of wording used in this argument is accepted. (See details)
}
  \item{burning}{
Logical. Whether or not standard deviation should be adapted. 
}
  \item{rotation}{
Logical. Whether or not an eigenvector rotation should be carried out.
}
  \item{iter}{
Numeric. The number of time \code{scamH} has been carried out so far. (See details)
}
  \item{...}{Parameters passed to other functions.}
}
\details{
For the user the main difference between \code{scamH} and \code{scamHT} is within the \code{data} and \code{start} arguments. As for \code{scamRandom}, it is specifically designed to estimate the parameter of the random effect and it is specifically it is heavily relying on the data and parameters available in the \code{data} and \code{start} arguments.

The algorithm implemented in this function assumes that the data follows a multivariate normal distribution. In that instance, it becomes valuable to check the distribution of the posterior to make sure that it is distributed following a pattern that look somewhat like a multivariate normal distribution. A way to do this is to use \code{\link{pairs}} to study the different models parameters (paramX, paramTr,paramRandom, means, sigma,...), see example in \code{\link{hmscH}}.

The \code{formula} was designed so that it is necessary to include the parameters and the descriptors. This function is thus capable of carrying out non-linear regression analyses. *** IMPORTANT : Although the function will run, the estimation of the parameters for a  non-linear model is very difficult and it is unlikely to converge unless many, many (many !!) iterations are carried out.*** 

Currently, two likelihood have been implemented, the binomial likelihood with a logit link, and the Poisson likelihood.

These functions were designed to converge to an accept ratio of 0.44.

These functions are constructed in such a way that if \code{burning} is set to \code{FALSE}, by default, \code{rotation} will be considered \code{FALSE} as well.

These functions are meant to be used within other functions such as \code{\link{hmscH}}. However, this help file was written for user who want to develop new approaches using this basic algorithm. 
}
\value{
  \item{start}{Object of class \code{HMSCparam}.}
  \item{rotation}{List with the following items:
		\describe{
			\item{\code{values}}{Matrix of eigenvalues after the update has been carried out. The matrix has the same dimension as \code{rotationVal}. If \code{rotation} was set to \code{FALSE}, the eigenvalues will be the same as the ones originally given to \code{scamH} (or \code{scamHT} or \code{scamRandom}).}
			\item{\code{vectors}}{A 3-dimensional array of eigenvectors after the update has been carried out. The array has the same dimension as \code{rotationVect}. If \code{rotation} was set to \code{FALSE}, the eigenvectors will be the same as the ones originally given to \code{scamH} (or \code{scamHT} or \code{scamRandom}).}
		}
	}
  For \code{scamH} and \code{scamHT}:
  \item{\code{sdvalue}}{An object of the same dimension as the argument \code{sdvalue}. It presents the standard deviations for associated to each of the X parameter after the update has been carried out. If \code{burning} was set to \code{FALSE}, \code{sdvalue} will be the same as the ones originally given to \code{scamH} or \code{scamHT} as an argument.}
  \item{\code{accept}}{An object of the same dimension as the argument \code{accept}. It presents the number of times each new regression parameter  has been accepted when estimating the parameters of X. If \code{burning} was set to \code{FALSE}, \code{accept} will be the same as the ones originally given to \code{scamH} or \code{scamHT} as an argument.}
 
  For \code{scamRandom}:
  \item{sdvalue}{A list of three component containing the updated standard deviations. This list includes exactly the same components as the ones first included in the function as argument.}
  \item{accept}{A list of three component containing the updated standard deviations. This list includes exactly the same components as the ones first included in the function as argument.}
}
\author{
F. Guillaume Blanchet
}
\note{
No example are given for this function because this function is meant to be ran within another function. As explained in the Details section above. 
}

\seealso{
\code{\link{hmscH}}, \code{\link{hmscHT}}, \code{\link{loglikelihood}}
}
\keyword{multivariate}
\keyword{regression}
\keyword{nonlinear}
